package cmd

import (
	"fmt"
	"log"
	"strconv"

	"github.com/spf13/cobra"
	"github.com/spf13/viper"
	ldap "gopkg.in/ldap.v2"
)

var (
	// user subcommand arguments
	flagUserUsername  string
	flagUserEmail     string
	flagUserFirstName string
	flagUserLastName  string
	flagUserIdNumber  int
)

func init() {
	// define add user subcommand flags
	addUserCmd.Flags().StringP("base_ou", "o", "", "LDAP OU to create the new user entry under")
	addUserCmd.Flags().StringP("uid_attribute", "a", "uid", "LDAP DN attribute for users")
	addUserCmd.Flags().StringP("home_directory", "d", "/home/%s", "User's home directory")
	addUserCmd.Flags().StringVarP(&flagUserUsername, "username", "u", "", "Username for a new user")
	addUserCmd.Flags().StringVarP(&flagUserEmail, "email", "e", "", "Email address for a new user")
	addUserCmd.Flags().StringVarP(&flagUserFirstName, "first_name", "f", "", "First name of a new user")
	addUserCmd.Flags().StringVarP(&flagUserLastName, "last_name", "l", "", "Last name of a new user")
	addUserCmd.Flags().IntVarP(&flagUserIdNumber, "id_number", "i", -1, "ID Number for a new user")
	// bind config file values to user flags
	viper.BindPFlag("user.base_ou", addUserCmd.Flags().Lookup("base_ou"))
	viper.BindPFlag("user.uid_attr", addUserCmd.Flags().Lookup("uid_attribute"))
	viper.BindPFlag("user.home_directory", addUserCmd.Flags().Lookup("home_directory"))

	// register add command and subcommands
	addCmd.AddCommand(addUserCmd)
	rootCmd.AddCommand(addCmd)
}

// define add command
var addCmd = &cobra.Command{
	Use:   "add",
	Short: "Add an LDAP resource to the directory",
	Long:  `Create and add an LDAP resource to your directory.`,
	Run: func(cmd *cobra.Command, args []string) {
		fmt.Println("Metaunix.net LDAP tool, add command. Available subcommands are: user")
	},
}

// define add user subcommand
var addUserCmd = &cobra.Command{
	Use:   "user",
	Short: "Add an LDAP user resource to the directory",
	Long:  `Create and add an LDAP user resource to your directory.`,
	Run: func(cmd *cobra.Command, args []string) {
		// create new LDAP connection
		l, err := ldap.Dial("tcp", fmt.Sprintf("%s:%d", viper.GetString("host"), viper.GetInt("port")))
		if err != nil {
			log.Fatal(err)
		}
		defer l.Close()

		// bind as the admin user
		err = l.Bind(viper.GetString("bind_dn"), viper.GetString("bind_pw"))
		if err != nil {
			log.Fatal(err)
		}

		// set up user attributes
		userDn := fmt.Sprintf("%s=%s,%s", viper.GetString("user.uid_attr"), flagUserUsername, viper.GetString("user.base_ou"))
		userDisplayName := fmt.Sprintf("%s %s", flagUserFirstName, flagUserLastName)
		userHome := fmt.Sprintf(viper.GetString("user.home_directory"), flagUserUsername)

		// create a new add request object
		addRequest := ldap.NewAddRequest(userDn)
		// add user attributes to the request
		addRequest.Attribute(viper.GetString("user.uid_attr"), []string{flagUserUsername})
		addRequest.Attribute("objectClass", viper.GetStringSlice("user.object_class"))
		addRequest.Attribute("mail", []string{flagUserEmail})
		addRequest.Attribute("givenName", []string{flagUserFirstName})
		addRequest.Attribute("sn", []string{flagUserLastName})
		addRequest.Attribute("cn", []string{flagUserFirstName})
		addRequest.Attribute("displayName", []string{userDisplayName})
		addRequest.Attribute("uidNumber", []string{strconv.Itoa(flagUserIdNumber)})
		addRequest.Attribute("gidNumber", []string{strconv.Itoa(flagUserIdNumber)})
		addRequest.Attribute("homeDirectory", []string{userHome})
		// loop through extra attributes
		for key, value := range viper.GetStringMapString("user.extra_attributes") {
			addRequest.Attribute(key, []string{value})
		}

		// perform the add operation
		err = l.Add(addRequest)
		if err != nil {
			log.Fatal(err)
		}
	},
}
